@@ -25,14 +25,9 @@ class Database:
 			self.addBar(instrument, bar, frequency)
 
 	def addBarsFromFeed(self, feed):
-		feed.start()
-		try:
-			for bars in feed:
-				if bars:
-					self.addBars(bars, feed.getFrequency())
-		finally:
-			feed.stop()
-			feed.join()
+		for dateTime, bars in feed:
+			if bars:
+				self.addBars(bars, feed.getFrequency())
 
 	def addBar(self, instrument, bar, frequency):
 		raise NotImplementedError()
